#!/usr/bin/env python
# encoding: utf-8
import cli_print as cp


def raw(s: str = ''):
    """
    Get input, retry if empty.

    :param str s: prompt
    :return: str
    """
    while True:
        value = input('> {}: '.format(s))
        if value:
            return value


def integer(s: str = ''):
    """
    Get a integer, retry if empty or ValueError.

    :param str s: prompt
    :return: int
    """
    value = None
    try:
        while True:
            value = raw(s)
            return int(value)
    except ValueError:
        print('"{}" is not a integer.'.format(value))
        return integer(s)


def confirm(s: str = ''):
    """
    Ask yes/no, retry if invalid.

    :param str s: prompt
    :return: bool
    """
    while True:
        value = input('> {} [y/n]: '.format(s)).lower()
        if value:
            if value in 'yesrtui':
                return True
            elif value in 'novbm,':
                return False


def pause(s: str = 'press any key to continue'):
    """
    Pause with a prompt.

    :param str s: prompt
    :return: None
    """
    _ = input('# {}: '.format(s))
    return


def copy(value: str, force: bool = False, key: str = None):
    """
    Copy to clip.

    :param str value: value to be copied
    :param bool force: without a prompt pause
    :param str key: tip
    :return: None
    """
    import cli_print as cp
    import pyperclip

    if not force:
        if key:
            pause('Press any key to copy {} '.format(key)
                  + cp.Fore.LIGHTCYAN_EX + str(value)
                  + cp.Style.RESET_ALL + ' to clip')
        else:
            pause('Press any key to copy '
                  + cp.Fore.LIGHTCYAN_EX + str(value)
                  + cp.Style.RESET_ALL + ' to clip')

        cp.wr('\x1b[1A\x1b[2K')  # remove the prompt line.
        cp.fi()
    if key:
        cp.about_t('Copy {}'.format(key), value, 'to clip')
    else:
        cp.about_t('Copy', value, 'to clip')
    pyperclip.copy(value)
    cp.success()
    return


def _print_sleep(i: int, t: int):
    """
    Function for `sleep`.

    :param int i: current
    :param int t: amount
    :return: None
    """
    if t < 90:
        cp.wr(cp.Fore.LIGHTBLUE_EX + ' - [' + '>' * i + '-' * (t - i) + '] sleep {}/{} s\r'.format(i, t))
        cp.fx()
    else:
        cp.wr(cp.Fore.LIGHTBLUE_EX + ' - sleep {}/{} s\r'.format(i, t))
        cp.fx()
    return


def sleep(t: int = None, b: int = None):
    """
    Sleep for a while.

    :param int t: Min seconds
    :param int b: Max seconds
    :return:
    """
    import time
    import random

    if t and b:
        t = random.randint(t, b)
    elif t is None:
        t = random.randint(10, 60)

    _print_sleep(0, t)

    for i in range(1, t + 1):
        time.sleep(1)
        cp.previous_line(True)
        _print_sleep(i, t)

    return t



#!/usr/bin/env python
# encoding: utf-8


def get_dict_by_keys(source, keys: list or dict, default_none: bool = True):
    """
    Get a dict, by keys.

    :param source:
        dict, json_string, or a object
    :param keys:
        list, or dict when there is expecting default values for keys
    :param bool default_none:
        Available only if keys is a list
        Set the default value is None, or ignore it, when the value does not exist
    :return:
        dict
    """
    r = dict()

    # dict
    if isinstance(source, dict):
        if isinstance(keys, list):
            for _key in keys:
                if _key in source.keys():
                    r[_key] = source[_key]
                else:
                    if default_none:
                        r[_key] = None
        elif isinstance(keys, dict):
            for _key, value in keys.items():
                if _key in source.keys():
                    r[_key] = source[_key]
                else:
                    r[_key] = value

    # json_string
    elif isinstance(source, str):
        try:
            import json
            source = json.loads(source)
        except:
            return r

        return get_dict_by_keys(source, keys=keys, default_none=default_none)

    # object
    else:
        if isinstance(keys, list):
            for _key in keys:
                if default_none:
                    r[_key] = getattr(source, _key, None)
                elif hasattr(source, _key):
                    r[_key] = getattr(source, _key)
        elif isinstance(keys, dict):
            for _key, value in keys.items():
                r[_key] = getattr(source, _key, value)

    return r



#!/usr/bin/env python
# encoding: utf-8

from . import dict_fn


def may_update_by_dict(row, row_dict: dict):
    """
    Update row by dict.

    :param row:
    :param dict row_dict:
    :return:
    """
    should_update = False

    for key, value in row_dict.items():
        if getattr(row, key) != value:
            setattr(row, key, value)
            should_update = True

    if should_update:
        row.save()

    return row


def may_update(row, source, keys):
    """
    On-demand Save

    :param row:     orm row
    :param source:  source dict/object
    :param keys:    keys/attributes
    :return:        orm row
    """
    row_dict = dict_fn.get_dict_by_keys(
        source=source,
        keys=keys,
    )

    return may_update_by_dict(row, row_dict)


def get_or_create_may_update(model, source, keys, defaults_extend: dict = None, **kwargs):
    """
    Get_or_create, may update

    :param model:       orm model
    :param source:      source dict/object
    :param keys:        keys/attributes
    :param defaults_extend:    default dict
    :param kwargs:      **kwargs
    :return:            orm row, created
    """
    row_dict = dict_fn.get_dict_by_keys(
        source=source,
        keys=keys,
    )

    defaults_extend = defaults_extend or dict()

    row, created = model.objects.get_or_create(defaults=dict(row_dict, **defaults_extend), **kwargs)

    if created:
        return row

    return may_update_by_dict(row, row_dict)



#!/usr/bin/env python
# encoding: utf-8

import os
import hashlib
import tempfile
import pickle


class FileCache:
    def __init__(self, filename_prefix: str = 'fc', cache_dir: str = None):
        """
        Init file cache.
        
        :param str filename_prefix: 
        :param str cache_dir: 
        """
        self._prefix = filename_prefix
        self._dir = cache_dir or tempfile.gettempdir()

    @staticmethod
    def __md5(s: str):
        """
        md5

        :param str s: source value
        :return: str
        """
        m = hashlib.md5()
        m.update(s.encode('utf-8'))
        return m.hexdigest()

    def __filename(self, key: str):
        """
        Generate full path to file via key.

        :param str key: key name
        :return: str
        """
        return os.path.join(self._dir, '{}_{}.cache'.format(self._prefix, self.__md5(key)))

    def put(self, key: str, data):
        """
        Cache a key to file.

        :param str key: key name
        :param data: value
        :return: str
        """
        path_to_file = self.__filename(key)

        with open(path_to_file, 'wb') as fp:
            pickle.dump(data, fp)

        return path_to_file

    def get(self, key: str, default=None):
        """
        Read cache from file.

        :param str key: key name
        :param default: default value
        :return:
        """
        path_to_file = self.__filename(key)

        if not os.path.exists(path_to_file):
            return default

        with open(path_to_file, 'rb') as fp:
            data = pickle.load(fp)

        return data

    def pull(self, key: str, default=None):
        """
        Get and forget.

        :param str key: key name
        :param default: default value
        :return:
        """
        data = self.get(key=key, default=default)
        self.forget(key=key)
        return data

    def forget(self, key: str):
        """
        Forget, remove the cache file.

        :param str key: key name
        :return: bool
        """
        path_to_file = self.__filename(key)

        if os.path.exists(path_to_file):
            os.remove(path_to_file)
            return True
        return False



#!/usr/bin/env python
# encoding: utf-8

import os


def read(path_to_file: str):
    """
    Read the file, and returns the content directly.

    :param str path_to_file: /path/to/file
    :return:
    """
    if os.path.exists(path_to_file):
        with open(path_to_file, 'r') as f:
            return f.read()

    import cli_print as cp
    cp.error('"{}" does not exist.'.format(path_to_file))
    return None


def get_list_from(path_to_file: str, comment: str = '#'):
    """
    Read the file, and returns a list of the valid lines.

    :param str path_to_file: /path/to/file
    :param str comment: Allow comment, ignore the lines which is starts with '#' as default
    :return: list
    """
    rows = list()

    if os.path.exists(path_to_file):
        for _line in open(path_to_file).readlines():
            _line = _line.strip()
            if _line:
                if comment:
                    if not _line.startswith(comment):
                        rows.append(_line)
                else:
                    rows.append(_line)

        return rows

    import cli_print as cp
    cp.error('"{}" does not exist.'.format(path_to_file))
    return rows


def read_to_list(path_to_file: str, comment: str = '#'):
    return get_list_from(path_to_file=path_to_file, comment=comment)


def put_list_to(path_to_file: str, lines: list):
    """
    Write a list to a file in lines.

    :param str path_to_file: /path/to/file
    :param list lines: source list
    :return: str
    """
    with open(path_to_file, 'w') as fp:
        content = list()
        for line in lines:
            content.append('{}'.format(line))
        fp.write('\n'.join(content))
    return path_to_file



#!/usr/bin/env python
# encoding: utf-8
import binascii
import sha3
from ecdsa import SigningKey, SECP256k1
from web3 import Web3


def generate_address_and_private_key():
    private = SigningKey.generate(curve=SECP256k1)
    public = private.get_verifying_key()

    keccak = sha3.keccak_256()
    keccak.update(public.to_string())

    address = Web3.toChecksumAddress('0x{}'.format(keccak.hexdigest()[24:]))
    private_key = binascii.hexlify(private.to_string()).decode()

    return address, private_key



#!/usr/bin/env python
# encoding: utf-8

import random


def salt(length: int = None,
         with_hint: bool = False,
         seed: str = '1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
         ):
    """
    Salt generator.

    :param int length: Length
    :param bool with_hint: Whether need a hint
    :param str seed: Seed
    :return: tuple or str
    """
    length = length or random.randint(8, 12)

    s = []
    for i in range(length):
        s.append(random.choice(seed))
    r = ''.join(s)

    if with_hint:
        return r, shrink_to_hint(r)
    else:
        return r


def shrink_to_hint(s: str):
    """
    Shrink a string to hint.

    :param str s: Source string
    :return: str
    """
    length = len(s)
    if length < 4:
        return '*' * length

    return '{}**{}'.format(s[0], s[length - 1])



#!/usr/bin/env python
# encoding: utf-8


def unique(source: list):
    """
    Unique.

    :param list source: Source list
    :return: list
    """
    r = []
    for el in source:
        if el not in r:
            r.append(el)
    return r


def strip(source: list):
    """
    Strip.

    :param list source: Source list
    :return: list
    """
    r = []
    for value in source:
        if isinstance(value, str):
            r.append(value.strip())
        else:
            r.append(value)
    return r


def remove(source: list, els=None):
    """
    Remove elements.

    :param list source: Source list
    :param els: Element(s) to be removed
    :return: list
    """
    r = []
    if els is None:
        els = ['', None]
    elif not isinstance(els, list):
        els = [els]

    for el in source:
        if el not in els:
            r.append(el)

    return r


def strip_and_unique(source: list):
    """
    Strip and unique.

    :param list source: Source list
    :return: list
    """
    r = unique(strip(source=source))
    return r


def strip_and_remove(source: list, els=None):
    """
    Strip and remove.

    :param list source: Source list
    :param els: Element(s) to be removed
    :return: list
    """
    r = remove(strip(source=source), els=els)
    return r


def remove_and_unique(source: list, els=None):
    """
    Remove and unique.

    :param list source: Source list
    :param els: Element(s) to be removed
    :return: list
    """
    r = unique(remove(source=source, els=els))
    return r


def sur(source: list, els=None):
    """
    Strip, unique, remove.

    :param list source: Source list
    :param els: Element(s) to be removed
    :return: list
    """
    r = remove(strip_and_unique(source=source), els=els)
    return r



#!/usr/bin/env python
# encoding: utf-8
import base64


def encode(s, urlsafe: bool = False):
    """
    Base64 encode.

    :param s:
    :param urlsafe: Whether use URL Safe Mode
    :return: str
    """
    if isinstance(s, str):
        s = s.encode('utf-8')

    if urlsafe:
        return base64.urlsafe_b64encode(s).decode('utf-8')

    return base64.b64encode(s).decode('utf-8')


def decode(s):
    """
    Base64 decode.

    :param s:
    :return: str
    """
    if isinstance(s, str):
        if len(s) % 4 > 0:
            s = s + '=' * (4 - len(s) % 4)
        s = s.encode('utf-8')

    # URL Safe
    s = s.translate(bytes.maketrans(b'-_', b'+/'))

    try:
        return base64.b64decode(s).decode('utf-8')
    except UnicodeDecodeError:
        return base64.b64decode(s)



#!/usr/bin/env python
# encoding: utf-8

import requests
import cli_print as cp


def random_an_user(gender: str = 'male', nation: str = 'us'):
    """
    Generate a fake user by https://randomuser.me/
    :param str gender: Gender
    :param str nation: Nation
    :return: dict
    """
    resp = requests.get('https://randomuser.me/api/?gender={gender}&nat={nation}'.format(
        gender=gender, nation=nation
    ))

    return resp.json()['results'][0]


def fake_name(gender: str = 'male', nation: str = 'us', cli: bool = False):
    """
    Get a random user first/last name.

    :param str gender: Gender
    :param str nation: Nation
    :param bool cli:
    :return: str, str
    """
    if cli:
        cp.getting('Random a name')

    r = random_an_user(gender=gender, nation=nation)

    name = r['name']

    first_name = name['first'].strip().capitalize()
    last_name = name['last'].strip().capitalize()

    if cli:
        cp.value('{} {}'.format(first_name, last_name))

    return first_name, last_name



#!/usr/bin/env python
# encoding: utf-8


def gram_align_right(s, line_width: int = 20):
    """
    Format for telegram, align right.

    :param s: input text
    :param int line_width: Width
    :return: str
    """
    return '`{}`'.format(str(s).rjust(line_width))



#!/usr/bin/env python
# encoding: utf-8

from setuptools import (setup, find_packages)

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name="qwert",
    version="2.1.6",
    packages=find_packages(),

    # metadata for upload to PyPI
    author="Vision Network",
    author_email="michael@vision.network",
    description="Python extensions: list/dict/file...",
    keywords='Python, Extensions, List, Dict, File',

    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/VisionNetworkProject/python-qwert",

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    install_requires=[
        'cli-print',
        'requests',
        'pyperclip',
        'web3',
    ],
)
